@@ -29,6 +29,7 @@
 #include "gmime-common.h"
 #include "gmime-object.h"
 #include "gmime-stream-mem.h"
+#include "gmime-events.h"
 #include "gmime-utils.h"
 
 
@@ -61,20 +62,21 @@ static void g_mime_object_class_init (GMimeObjectClass *klass);
 static void g_mime_object_init (GMimeObject *object, GMimeObjectClass *klass);
 static void g_mime_object_finalize (GObject *object);
 
-static void prepend_header (GMimeObject *object, const char *name, const char *value);
-static void append_header (GMimeObject *object, const char *name, const char *value);
-static void set_header (GMimeObject *object, const char *name, const char *value);
-static const char *get_header (GMimeObject *object, const char *name);
-static gboolean remove_header (GMimeObject *object, const char *name);
-static void set_content_type (GMimeObject *object, GMimeContentType *content_type);
-static char *get_headers (GMimeObject *object);
-static ssize_t write_to_stream (GMimeObject *object, GMimeStream *stream);
+static void object_prepend_header (GMimeObject *object, const char *name, const char *value);
+static void object_append_header (GMimeObject *object, const char *name, const char *value);
+static void object_set_header (GMimeObject *object, const char *name, const char *value);
+static const char *object_get_header (GMimeObject *object, const char *name);
+static gboolean object_remove_header (GMimeObject *object, const char *name);
+static void object_set_content_type (GMimeObject *object, GMimeContentType *content_type);
+static char *object_get_headers (GMimeObject *object);
+static ssize_t object_write_to_stream (GMimeObject *object, GMimeStream *stream);
+static void object_encode (GMimeObject *object, GMimeEncodingConstraint constraint);
 
 static ssize_t write_content_type (GMimeStream *stream, const char *name, const char *value);
 static ssize_t write_disposition (GMimeStream *stream, const char *name, const char *value);
 
-static void content_type_changed (GMimeContentType *content_type, GMimeObject *object);
-static void content_disposition_changed (GMimeContentDisposition *disposition, GMimeObject *object);
+static void content_type_changed (GMimeContentType *content_type, gpointer args, GMimeObject *object);
+static void content_disposition_changed (GMimeContentDisposition *disposition, gpointer args, GMimeObject *object);
 
 static void type_registry_init (void);
 
@@ -119,14 +121,15 @@ g_mime_object_class_init (GMimeObjectClass *klass)
 	
 	object_class->finalize = g_mime_object_finalize;
 	
-	klass->prepend_header = prepend_header;
-	klass->append_header = append_header;
-	klass->remove_header = remove_header;
-	klass->set_header = set_header;
-	klass->get_header = get_header;
-	klass->set_content_type = set_content_type;
-	klass->get_headers = get_headers;
-	klass->write_to_stream = write_to_stream;
+	klass->prepend_header = object_prepend_header;
+	klass->append_header = object_append_header;
+	klass->remove_header = object_remove_header;
+	klass->set_header = object_set_header;
+	klass->get_header = object_get_header;
+	klass->set_content_type = object_set_content_type;
+	klass->get_headers = object_get_headers;
+	klass->write_to_stream = object_write_to_stream;
+	klass->encode = object_encode;
 	
 	type_registry_init ();
 }
@@ -149,16 +152,12 @@ g_mime_object_finalize (GObject *object)
 	GMimeObject *mime = (GMimeObject *) object;
 	
 	if (mime->content_type) {
-		g_signal_handlers_disconnect_matched (mime->content_type,
-						      G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA,
-						      0, 0, NULL, content_type_changed, object);
+		g_mime_event_remove (mime->content_type->priv, (GMimeEventCallback) content_type_changed, object);
 		g_object_unref (mime->content_type);
 	}
 	
 	if (mime->disposition) {
-		g_signal_handlers_disconnect_matched (mime->disposition,
-						      G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA,
-						      0, 0, NULL, content_disposition_changed, object);
+		g_mime_event_remove (mime->disposition->priv, (GMimeEventCallback) content_disposition_changed, object);
 		g_object_unref (mime->disposition);
 	}
 	
@@ -198,7 +197,7 @@ write_content_type (GMimeStream *stream, const char *name, const char *value)
 }
 
 static void
-content_type_changed (GMimeContentType *content_type, GMimeObject *object)
+content_type_changed (GMimeContentType *content_type, gpointer args, GMimeObject *object)
 {
 	GMimeParam *params;
 	GString *string;
@@ -244,7 +243,7 @@ write_disposition (GMimeStream *stream, const char *name, const char *value)
 }
 
 static void
-content_disposition_changed (GMimeContentDisposition *disposition, GMimeObject *object)
+content_disposition_changed (GMimeContentDisposition *disposition, gpointer args, GMimeObject *object)
 {
 	char *str;
 	
@@ -405,16 +404,14 @@ g_mime_object_new_type (const char *type, const char *subtype)
 
 
 static void
-set_content_type (GMimeObject *object, GMimeContentType *content_type)
+object_set_content_type (GMimeObject *object, GMimeContentType *content_type)
 {
 	if (object->content_type) {
-		g_signal_handlers_disconnect_matched (object->content_type,
-						      G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA,
-						      0, 0, NULL, content_type_changed, object);
+		g_mime_event_remove (object->content_type->priv, (GMimeEventCallback) content_type_changed, object);
 		g_object_unref (object->content_type);
 	}
 	
-	g_signal_connect (content_type, "changed", G_CALLBACK (content_type_changed), object);
+	g_mime_event_add (content_type->priv, (GMimeEventCallback) content_type_changed, object);
 	object->content_type = content_type;
 	g_object_ref (content_type);
 }
@@ -456,7 +453,7 @@ g_mime_object_set_content_type (GMimeObject *object, GMimeContentType *content_t
 	
 	GMIME_OBJECT_GET_CLASS (object)->set_content_type (object, content_type);
 	
-	content_type_changed (content_type, object);
+	content_type_changed (content_type, NULL, object);
 }
 
 
@@ -548,13 +545,11 @@ static void
 _g_mime_object_set_content_disposition (GMimeObject *object, GMimeContentDisposition *disposition)
 {
 	if (object->disposition) {
-		g_signal_handlers_disconnect_matched (object->disposition,
-						      G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA,
-						      0, 0, NULL, content_disposition_changed, object);
+		g_mime_event_remove (object->disposition->priv, (GMimeEventCallback) content_disposition_changed, object);
 		g_object_unref (object->disposition);
 	}
 	
-	g_signal_connect (disposition, "changed", G_CALLBACK (content_disposition_changed), object);
+	g_mime_event_add (disposition->priv, (GMimeEventCallback) content_disposition_changed, object);
 	object->disposition = disposition;
 	g_object_ref (disposition);
 }
@@ -579,7 +574,7 @@ g_mime_object_set_content_disposition (GMimeObject *object, GMimeContentDisposit
 	
 	_g_mime_object_set_content_disposition (object, disposition);
 	
-	content_disposition_changed (disposition, object);
+	content_disposition_changed (disposition, NULL, object);
 }
 
 
@@ -609,7 +604,7 @@ g_mime_object_set_disposition (GMimeObject *object, const char *disposition)
 	disp = g_mime_content_disposition_new ();
 	g_mime_content_disposition_set_disposition (disp, disposition);
 	g_mime_object_set_content_disposition (object, disp);
-	g_object_ref (disp);
+	g_object_unref (disp);
 }
 
 
@@ -741,7 +736,7 @@ process_header (GMimeObject *object, const char *header, const char *value)
 {
 	GMimeContentDisposition *disposition;
 	GMimeContentType *content_type;
-	int i;
+	guint i;
 	
 	for (i = 0; i < G_N_ELEMENTS (content_headers); i++) {
 		if (!g_ascii_strcasecmp (content_headers[i], header))
@@ -773,7 +768,7 @@ process_header (GMimeObject *object, const char *header, const char *value)
 }
 
 static void
-prepend_header (GMimeObject *object, const char *header, const char *value)
+object_prepend_header (GMimeObject *object, const char *header, const char *value)
 {
 	if (!process_header (object, header, value))
 		g_mime_header_list_prepend (object->headers, header, value);
@@ -799,7 +794,7 @@ g_mime_object_prepend_header (GMimeObject *object, const char *header, const cha
 }
 
 static void
-append_header (GMimeObject *object, const char *header, const char *value)
+object_append_header (GMimeObject *object, const char *header, const char *value)
 {
 	if (!process_header (object, header, value))
 		g_mime_header_list_append (object->headers, header, value);
@@ -826,7 +821,7 @@ g_mime_object_append_header (GMimeObject *object, const char *header, const char
 
 
 static void
-set_header (GMimeObject *object, const char *header, const char *value)
+object_set_header (GMimeObject *object, const char *header, const char *value)
 {
 	if (!process_header (object, header, value))
 		g_mime_header_list_set (object->headers, header, value);
@@ -853,7 +848,7 @@ g_mime_object_set_header (GMimeObject *object, const char *header, const char *v
 
 
 static const char *
-get_header (GMimeObject *object, const char *header)
+object_get_header (GMimeObject *object, const char *header)
 {
 	return g_mime_header_list_get (object->headers, header);
 }
@@ -881,9 +876,9 @@ g_mime_object_get_header (GMimeObject *object, const char *header)
 
 
 static gboolean
-remove_header (GMimeObject *object, const char *header)
+object_remove_header (GMimeObject *object, const char *header)
 {
-	int i;
+	guint i;
 	
 	for (i = 0; i < G_N_ELEMENTS (content_headers); i++) {
 		if (!g_ascii_strcasecmp (content_headers[i], header))
@@ -893,9 +888,7 @@ remove_header (GMimeObject *object, const char *header)
 	switch (i) {
 	case HEADER_CONTENT_DISPOSITION:
 		if (object->disposition) {
-			g_signal_handlers_disconnect_matched (object->disposition,
-							      G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA,
-							      0, 0, NULL, content_disposition_changed, object);
+			g_mime_event_remove (object->disposition->priv, (GMimeEventCallback) content_disposition_changed, object);
 			g_object_unref (object->disposition);
 			object->disposition = NULL;
 		}
@@ -936,7 +929,7 @@ g_mime_object_remove_header (GMimeObject *object, const char *header)
 
 
 static char *
-get_headers (GMimeObject *object)
+object_get_headers (GMimeObject *object)
 {
 	return g_mime_header_list_to_string (object->headers);
 }
@@ -961,7 +954,7 @@ g_mime_object_get_headers (GMimeObject *object)
 
 
 static ssize_t
-write_to_stream (GMimeObject *object, GMimeStream *stream)
+object_write_to_stream (GMimeObject *object, GMimeStream *stream)
 {
 	return -1;
 }
@@ -986,6 +979,31 @@ g_mime_object_write_to_stream (GMimeObject *object, GMimeStream *stream)
 }
 
 
+static void
+object_encode (GMimeObject *object, GMimeEncodingConstraint constraint)
+{
+	return;
+}
+
+
+/**
+ * g_mime_object_encode:
+ * @object: a #GMimeObject
+ * @constraint: a #GMimeEncodingConstraint
+ *
+ * Calculates and sets the most efficient Content-Transfer-Encoding
+ * for this #GMimeObject and all child parts based on the @constraint
+ * provided.
+ **/
+void
+g_mime_object_encode (GMimeObject *object, GMimeEncodingConstraint constraint)
+{
+	g_return_if_fail (GMIME_IS_OBJECT (object));
+	
+	return GMIME_OBJECT_GET_CLASS (object)->encode (object, constraint);
+}
+
+
 /**
  * g_mime_object_to_string:
  * @object: a #GMimeObject
