@@ -23,6 +23,7 @@
 #define __GMIME_H__
 
 #include <glib.h>
+#include <gmime/gmime-version.h>
 #include <gmime/gmime-error.h>
 #include <gmime/gmime-charset.h>
 #include <gmime/gmime-iconv.h>
@@ -49,9 +50,11 @@
 #include <gmime/gmime-stream-file.h>
 #include <gmime/gmime-stream-filter.h>
 #include <gmime/gmime-stream-fs.h>
+#include <gmime/gmime-stream-gio.h>
 #include <gmime/gmime-stream-mem.h>
 #include <gmime/gmime-stream-mmap.h>
 #include <gmime/gmime-stream-null.h>
+#include <gmime/gmime-stream-pipe.h>
 #include <gmime/gmime-filter.h>
 #include <gmime/gmime-filter-basic.h>
 #include <gmime/gmime-filter-best.h>
@@ -65,8 +68,6 @@
 #include <gmime/gmime-filter-strip.h>
 #include <gmime/gmime-filter-windows.h>
 #include <gmime/gmime-filter-yenc.h>
-#include <gmime/gmime-session.h>
-#include <gmime/gmime-session-simple.h>
 #include <gmime/gmime-cipher-context.h>
 #include <gmime/gmime-gpg-context.h>
 
@@ -109,17 +110,6 @@ extern const guint gmime_interface_age;
  **/
 extern const guint gmime_binary_age;
 
-
-/**
- * GMIME_CHECK_VERSION:
- * @major: Minimum major version
- * @minor: Minimum minor version
- * @micro: Minimum micro version
- *
- * Macro that just calls g_mime_check_version()
- **/
-#define GMIME_CHECK_VERSION(major,minor,micro) g_mime_check_version (major, minor, micro)
-
 gboolean g_mime_check_version (guint major, guint minor, guint micro);
 
 
