@@ -24,9 +24,9 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <unistd.h>
 #include <fcntl.h>
 #include <time.h>
 
@@ -94,7 +94,6 @@ test_parser (GMimeStream *stream)
 {
 	GMimeParser *parser;
 	GMimeMessage *message;
-	gboolean is_html;
 	char *text;
 	
 	fprintf (stdout, "\nTesting MIME parser...\n\n");
@@ -130,11 +129,11 @@ test_parser (GMimeStream *stream)
 	{
 		GMimeStream *stream;
 		
-		g_mime_header_list_set_raw (GMIME_OBJECT (message)->headers, NULL);
+		g_mime_header_list_set_stream (GMIME_OBJECT (message)->headers, NULL);
 		
 		fprintf (stdout, "\nTesting preservation of headers...\n\n");
 		stream = g_mime_stream_file_new (stdout);
-		/*g_mime_header_set_raw (GMIME_OBJECT (message)->headers, NULL);*/
+		/*g_mime_header_set_stream (GMIME_OBJECT (message)->headers, NULL);*/
 		g_mime_header_list_write_to_stream (GMIME_OBJECT (message)->headers, stream);
 		g_mime_stream_flush (stream);
 		GMIME_STREAM_FILE (stream)->fp = NULL;
@@ -179,7 +178,7 @@ int main (int argc, char **argv)
 	else
 		return 0;
 	
-	if ((fd = open (filename, O_RDONLY)) == -1)
+	if ((fd = open (filename, O_RDONLY, 0)) == -1)
 		return 0;
 	
 #ifdef STREAM_MMAP
